From 6bc8e8395e57c2bd640edcfc603cb89ddd4b5774 Mon Sep 17 00:00:00 2001
From: Feraidoon Mehri <feraidoonmehri@gmail.com>
Date: Sat, 22 Oct 2022 14:19:24 +0330
Subject: [PATCH] ox.el: Refactor variable org-html--id-attr-prefix,
 ox-html.el: Add support for the ID property to org-html--reference

* org/ox.el (org-html--id-attr-prefix): Refactor hardcoded "ID-" as a
new private variable.

* org/ox-html.el (org-html--reference): Add support for the ID
property (previously only supported CUSTOM_ID)

These changes make crossfile ID links work in the exported HTML when
`org-html-prefer-user-labels' is not-nil.
---
 lisp/org/ox-html.el | 39 ++++++++++++++++++++++++++-------------
 lisp/org/ox-odt.el  |  2 +-
 lisp/org/ox.el      |  3 +++
 3 files changed, 30 insertions(+), 14 deletions(-)

diff --git a/lisp/org/ox-html.el b/lisp/org/ox-html.el
index 9cf9125aeb..77fd66d711 100644
--- a/lisp/org/ox-html.el
+++ b/lisp/org/ox-html.el
@@ -1653,22 +1653,35 @@ When NAMED-ONLY is non-nil and DATUM has no NAME keyword, return
 nil.  This doesn't apply to headlines, inline tasks, radio
 targets and targets."
   (let* ((type (org-element-type datum))
-	 (user-label
-	  (org-element-property
-	   (pcase type
-	     ((or `headline `inlinetask) :CUSTOM_ID)
-	     ((or `radio-target `target) :value)
-	     (_ :name))
-	   datum)))
+         (custom-id-p nil)
+         (user-label
+          (or
+           (org-element-property
+            (pcase type
+              ((or `headline `inlinetask)
+               (progn
+                 (setq custom-id-p t)
+                 :CUSTOM_ID))
+              ((or `radio-target `target) :value)
+              (_ :name))
+            datum)
+           (if-let
+               ((id-property (org-element-property
+                              :ID
+                              datum)))
+               (progn
+                 (setq custom-id-p nil)
+                 (concat org-html--id-attr-prefix id-property))))))
+
     (cond
      ((and user-label
-	   (or (plist-get info :html-prefer-user-labels)
-	       ;; Used CUSTOM_ID property unconditionally.
-	       (memq type '(headline inlinetask))))
+           (or (plist-get info :html-prefer-user-labels)
+               ;; Used CUSTOM_ID property unconditionally.
+               custom-id-p))
       user-label)
      ((and named-only
-	   (not (memq type '(headline inlinetask radio-target target)))
-	   (not user-label))
+           (not (memq type '(headline inlinetask radio-target target)))
+           (not user-label))
       nil)
      (t
       (org-export-get-reference datum info)))))
@@ -3149,7 +3162,7 @@ INFO is a plist holding contextual information.  See
 	(pcase (org-element-type destination)
 	  ;; ID link points to an external file.
 	  (`plain-text
-	   (let ((fragment (concat "ID-" path))
+	   (let ((fragment (concat org-html--id-attr-prefix path))
 		 ;; Treat links to ".org" files as ".html", if needed.
 		 (path (funcall link-org-files-as-html-maybe
 				destination info)))
diff --git a/lisp/org/ox-odt.el b/lisp/org/ox-odt.el
index 7f2e8ba47f..d2c14237ac 100644
--- a/lisp/org/ox-odt.el
+++ b/lisp/org/ox-odt.el
@@ -1802,7 +1802,7 @@ holding contextual information."
 	   ;; Extra targets.
 	   (extra-targets
 	    (let ((id (org-element-property :ID headline)))
-	      (if id (org-odt--target "" (concat "ID-" id)) "")))
+	      (if id (org-odt--target "" (concat org-html--id-attr-prefix id)) "")))
 	   ;; Title.
 	   (anchored-title (org-odt--target full-text id)))
       (cond
diff --git a/lisp/org/ox.el b/lisp/org/ox.el
index 56bb4b74df..cc67c25e35 100644
--- a/lisp/org/ox.el
+++ b/lisp/org/ox.el
@@ -269,6 +269,9 @@ When this copy is created, all Org related local variables not in
 this list are copied to the new buffer.  Variables with an
 unreadable value are also ignored.")
 
+(defvar org-html--id-attr-prefix "ID-"
+  "Prefix to use in ID attributes of exported HTML elements when the ID is being determined from the ID property.")
+
 (defvar org-export-async-debug nil
   "Non-nil means asynchronous export process should leave data behind.
 
-- 
2.33.1

